@@ -148,6 +148,18 @@ class Catalog:
         #Return the list of channels matching the substring
         return channels_matching_substring
     
+    #Get a single channel matching a provided string, return none if not found
+    def get_channel(self, channel):
+        #Find the channel, and if not found, return none
+        retrieved_channel = None
+        for ch in self.channels:
+            if ch['name'] == channel:
+                retrieved_channel = ch
+                break
+        
+        #If it is found, return the channel
+        return retrieved_channel
+    
     def get_bundles(self):
         return self.bundles
 
@@ -164,6 +176,39 @@ class Catalog:
         
         #Return the list of bundles matching the substring
         return bundles_matching_substring
+    
+    #Get a single bundle matching a provided string, return none if not found
+    def get_bundle(self, bundle):
+        #Find the bundle, and if not found, return none
+        retrieved_bundle = None
+        for b in self.bundles:
+            if b['name'] == bundle:
+                retrieved_bundle = b
+                break
+        
+        #If it is found, return the channel
+        return retrieved_bundle
+
+    #Function returns a formatted dict containing a channel and its corresponding bundles
+    #The output of this function can be plugged into the add_channel_and_bundles function
+    #in order to add a channel and its corresponding bundles from one catalog to another
+    def get_channel_and_bundles(self, channel):
+        #Get the channel matching the provided channel name
+        channel = self.get_channel(channel)
+
+        #If channel is not found then return none
+        if channel == None:
+            return None
+        
+        #If channel exists, then retrieve all its bundles
+        bundles = []
+        for entry in channel['entries']:
+            bundle = self.get_bundle(entry['name'])
+            if bundle != None:
+                bundles.append(bundle)
+        
+        #After retrieving all bundles corresponding to the channel, return the formatted dict
+        return { 'channel' : channel, 'bundles' : bundles }
 
     def get_default_channel(self):
         return self.package['defaultChannel']
@@ -281,7 +326,44 @@ class Catalog:
                 if 'entries' in c:
                     c['entries'] = []
                 c['entries'].append(data)
-                
+    
+    #This function adds a channel and bundles from a formatted dict that is provided as a parameter
+    #It expects the same format as is returned by the get_channel_and_bundles function
+    def add_channel_and_bundles(self, channel_and_bundles):
+        #Sanity check on the channel and bundles input parameters
+        if channel_and_bundles == None: #Added since the get_channel_and_bundles function can return None now
+            raise CatalogError("The provided channel and bundles parameter was None")
+        if 'channel' not in channel_and_bundles.keys():
+            raise CatalogError("No channel was provided")
+        if 'name' not in channel_and_bundles['channel'].keys():
+            raise CatalogError("The provided channel was unnamed")
+        if 'bundles' not in channel_and_bundles.keys():
+            raise CatalogError("No bundles were provided")
+        
+        #Sanity check on channel before appending
+        for channel in self.channels:
+            if channel['name'] == channel_and_bundles['channel']['name']:
+                raise CatalogError(f"Channel already exists: {channel_and_bundles['channel']['name']}")
+        
+        #Add the channel to the catalog along with its entries
+        self.channels.append(channel_and_bundles['channel'])
+
+        #Add the bundles corresponding to the channel entries into the catalog
+        for bundle in channel_and_bundles['bundles']:
+            #Sanity check that the bundle is named
+            if 'name' not in bundle.keys():
+                raise CatalogError("A provided bundle was unnamed")
+            
+            #Sanity check that the bundle doesn't already exist
+            bundle_exists = False
+            for b in self.bundles:
+                if b['name'] == bundle['name']:
+                    bundle_exists = True
+            
+            #If it doesn't already exist, then append it
+            if not bundle_exists:
+                self.bundles.append(bundle)
+
     def get_latest_channel_entry(self, channel):
         names = [e['name'] for e in channel['entries'] ]
         names = natsorted(names)
